package CompAnalResultSource::SeqDBResultSource::SeqDBResultHandler;

use strict;
use SOAP::Lite on_fault => sub { my ($soap, $res) = @_;
			     print ref $res ? $res->faultstring : $soap->transport->status, "\n"};

use CompAnalResultSource;

my $seqdbservice=undef;
my $orgservice=undef;
my $resultservice=undef;

BEGIN {
    my $mugen=$PreferencesHandler::Preferences->getDocumentElement();
    my @nodes=$mugen->getElementsByTagName('resultdatasources');
    my $rds=shift @nodes;
    my @nodes=$rds->getElementsByTagName('seqdbresults',0);
    if (scalar(@nodes)==0) {
	my $doctype=$PreferencesHandler::Preferences->getDoctype();
	$doctype->addElementDecl('seqdbresults','EMPTY');
	$doctype->addAttDef('seqdbresults','url','CDATA','#REQUIRED');
	my $seqdb=$PreferencesHandler::Preferences->createElement('seqdbresults');
	$seqdb->setAttribute('url','localhost:9090');
	$rds->appendChild($seqdb);
    }
}

sub init {

    my $res=0;
    if (!$seqdbservice || !$orgservice || !$resultservice) {
	my @nodes=$PreferencesHandler::Preferences->getElementsByTagName('seqdbresults');
	my $seqdb=shift @nodes;
	if ($seqdb) {
	    my $url=$seqdb->getAttribute('url');
	    eval { 
		$seqdbservice=new SOAP::Lite->service($url."/Seqdb_service?WSDL");
		$orgservice=SOAP::Lite->service($url."/Organism_service?WSDL");
		$resultservice=SOAP::Lite->service($url."/Result_service?WSDL");
	    };
	
	    return 0
		if (!$seqdbservice || !$orgservice || !$resultservice);
	
	    $seqdbservice->xmlschema(2001);
	    $seqdbservice->maptype->{Organism}='http://Seqdb.org/types/Seqdb_service';
	    $seqdbservice->maptype->{ArrayOfOrganism}='http://Seqdb.org/types/Seqdb_service';
	    $seqdbservice->maptype->{ArrayOfstring}='http://Seqdb.org/types/Seqdb_service';
	    
#init du service Organism
	    $orgservice->xmlschema(2001);
	    $orgservice->serializer(new AlphaSerializer());
	    $orgservice->maptype->{Organism}='http://Seqdb.org/types/Organism_service';
	    $orgservice->maptype->{ArrayOfOrganism}='http://Seqdb.org/types/Organism_service';
	    $orgservice->maptype->{Element}='http://Seqdb.org/types/Organism_service';
	    $orgservice->maptype->{ArrayOfElement}='http://Seqdb.org/types/Organism_service';
	    $orgservice->maptype->{Gene}='http://Seqdb.org/types/Organism_service';
	    $orgservice->maptype->{ArrayOfGene}='http://Seqdb.org/types/Organism_service';
	    
#init du service Result 
	    $resultservice->xmlschema(2001);
	    $resultservice->serializer(new AlphaSerializer());
	    $resultservice->maptype->{Element}='http://Seqdb.org/types/Result_service';
	    $resultservice->maptype->{ArrayOfElement}='http://Seqdb.org/types/Result_service';
	    $resultservice->maptype->{Result}='http://Seqdb.org/types/Result_service';
	    $resultservice->maptype->{ArrayOfResult}='http://Seqdb.org/types/Result_service';
	    $resultservice->maptype->{ResultType}='http://Seqdb.org/types/Result_service';
	    $resultservice->maptype->{ArrayOfResultType}='http://Seqdb.org/types/Result_service';
	}
	
    }

    $res=1
	if ($seqdbservice && $orgservice && $resultservice);
	
    return $res;
}

sub getOrganisms{
    my $self=shift;
    init unless $seqdbservice;
    my $organisms=$seqdbservice->getOrganisms();
    return $organisms;
} 

sub getElements{
    my $self=shift;
    my $organisms=shift;
    init unless $orgservice;

    my @organismstab=();
   foreach my $org (@$organisms) {
       push @organismstab, new Organism($org);
    }
    my $SOAPorgtab=SOAP::Data->type('ArrayOfOrganism')->value(\@organismstab);
    my $elements=$orgservice->getElementsByOrganisms($SOAPorgtab);
    return $elements;
}

sub getExistingResults{
    my $self=shift;
   
    init unless $resultservice;
    
    my $results=undef;
  
    $results = $resultservice->getTypeResults();
  
    return $results;
}

sub getResultsFromElements{
    my $self=shift;
    my $elements=shift;
   
    init unless $resultservice;
    
    my $results=undef;
    my @elementstab=();
    
    foreach my $elt (@$elements) {
       push @elementstab, new Element($elt);
    }
    
    my $SOAPelttab=SOAP::Data->type('ArrayOfElement')->value(\@elementstab);

    $results = $resultservice->getResultsByElements($SOAPelttab);
    
    return $results;

}

sub getAtypicalGenes{
    my $self=shift;
    my $element=shift;
    init unless $resultservice;
    my $SOAPelement=new Element($element);
    my $genesWS=$resultservice->getAtypicalGenesByElement($SOAPelement);
    return $genesWS;
}


###Alphaserializer####################################################################
#####################################################################################
package AlphaSerializer;

@AlphaSerializer::ISA='SOAP::Serializer';


# Mthode de srialization.
# Pour les instances des autres classes  srialiser, il suffit de recopier le contenu
# de cette mthode dans une mthode appele :
#              as_MachinChose
# Pour des classes dans des sous-packages (ex.: MonPackage::MaClasse), il faut remplacer
# les :: par des __, ce qui donnerait :
#              as_MonPackage__MaClasse
sub _alphamapper{
    my $self=shift;
    my ($value, $name, $type, $attr) = @_;

    return [
	    $name || $self->SUPER::gen_name(),
	    {%$attr, 'xsi:type' => $self->maptypetouri($type)},
	    [map {$self->encode_object($value->{$_}, $_)} sort { $a cmp $b } keys %$value ],
	    $self->gen_id($value)
	    ];
}
sub as_Organism {
    return (_alphamapper( @_ ) );
}

sub as_Element {
    return (_alphamapper( @_ ) );
   
}

sub as_Gene {
    return (_alphamapper( @_ ) );
   
}
sub as_Result {
    return (_alphamapper( @_ ) );
   
}
sub as_ResultType {
    return (_alphamapper( @_ ) );
   
}


# Pas sur quelle marche car concidere comme deux Complexestypes gene et geneWS.
sub as_GeneWithState {
    return (_alphamapper( @_ ) );

}

# Pas sur quelle marche car concidere comme deux Complexestypes elt et eltWS.
sub as_ElementWithState {
    return (_alphamapper( @_ ) );

}

#######Empequatage SOAP#######################################################################
#############################################################################################
package Organism;

sub  new {
    my $class=shift;
    my $organism=shift;
   
    my $self={
	name=>SOAP::Data->type('string')->value($organism->{name}),
	organism_Id=>SOAP::Data->type('string')->value($organism->{organism_Id}),	
	shortName=>SOAP::Data->type('string')->value($organism->{shortName})
	};
    bless $self,$class;
    return $self ;
}

package Element;

sub new {
    my $class=shift;
    my $element=shift;
   
    my $self={
	accession=>SOAP::Data->type('string')->value($element->{accession}),
	description=>SOAP::Data->type('string')->value($element->{description}),
	element_Id=>SOAP::Data->type('string')->value($element->{element_Id}),
	organism_Id=>SOAP::Data->type('string')->value($element->{organism_Id}),
	taille=>SOAP::Data->type('string')->value($element->{taille}),
	type=>SOAP::Data->type('string')->value($element->{type})
	};
    return bless $self,$class;
}

package ElementWithState;

sub new {
    my $class=shift;
    my $element=shift;
   
    my $self={
	accession=>SOAP::Data->type('string')->value($element->{accession}),
	description=>SOAP::Data->type('string')->value($element->{description}),
	element_Id=>SOAP::Data->type('string')->value($element->{element_Id}),
	organism_Id=>SOAP::Data->type('string')->value($element->{organism_Id}),
	taille=>SOAP::Data->type('string')->value($element->{taille}),
	type=>SOAP::Data->type('string')->value($element->{type}),
	geneRAtypical=>SOAP::Data->type('float')->value($element->{geneRAtypical}),
	geneRForward=>SOAP::Data->type('float')->value($element->{geneRForward}),
	geneRReverse=>SOAP::Data->type('float')->value($element->{geneRReverse})
	};
    return bless $self,$class;
}
package Gene;

sub new {
    my $class=shift;
    my $gene=shift;
    
    my $self={	
	element_Id=>SOAP::Data->type('string')->value($gene->{element_Id}),
	gene_Id=>SOAP::Data->type('string')->value($gene->{organism_Id}),
	name=>SOAP::Data->type('string')->value($gene->{name}),
	start=>SOAP::Data->type('int')->value($gene->{start}),
	stop=>SOAP::Data->type('int')->value($gene->{stop}),
	strand=>SOAP::Data->type('int')->value($gene->{strand})
    };
    return bless $self,$class;
}
package GeneWithState;

sub new {
    my $class=shift; 
    my $gene=shift;
    
    my $self={	
	element_Id=>SOAP::Data->type('string')->value($gene->{element_Id}),
	gene_Id=>SOAP::Data->type('string')->value($gene->{organism_Id}),
	name=>SOAP::Data->type('string')->value($gene->{name}),
	start=>SOAP::Data->type('int')->value($gene->{start}),
	stop=>SOAP::Data->type('int')->value($gene->{stop}),
	strand=>SOAP::Data->type('int')->value($gene->{strand}),
	state=>SOAP::Data->type('string')->value($gene->{state})
    };
    return bless $self,$class;
}
package Result;

sub new {
    my $class=shift; 
    my $result=shift;
    
    my $self={	
	element_Id=>SOAP::Data->type('string')->value($result->{element_Id}),
	resulttype_Id=>SOAP::Data->type('string')->value($result->{resultType_Id}),
    };
    return bless $self,$class;
}
package ResultType;

sub new {
    my $class=shift; 
    my $result=shift;
    
    my $self={	
	description=>SOAP::Data->type('string')->value($result->{description}),
	resulttype_Id=>SOAP::Data->type('string')->value($result->{resultType_Id}),
    };
    return bless $self,$class;
}

1;
