package CompAnalResultSourceWindow::SeqDBResultsWindow; 

use strict;

use CompAnalResultSourceWindow;

@CompAnalResultSourceWindow::SeqDBResultsWindow::ISA=qw(Gtk::Dialog CompAnalResultSourceWindow);

my $false = 0;
my $true = 1;

sub new{
    my $class=shift;
    my $manager=shift;

    my $self=new Gtk::Dialog();
    $self->set_title("Results from SeqDB");
    $self->set_modal( $true );
    $self->set_default_size( 400, 400 );
    $self->set_position('center');
	
    $self->{manager}=$manager;

    _load_leaves_trees( $self);
    $self->{tree_organisms}= _create_tree_organisms($self,"/Organisms");
    $self->{tree_results} = _create_tree_results($self,"/Results");
  
      
    my $notbook_vbox=$self->vbox();
    
    my $notebook = new Gtk::Notebook();
    $notbook_vbox->pack_start($notebook,$true,$true,0);

    my $tree_frame= new Gtk::Frame();
    my $tree_window = new Gtk::ScrolledWindow( undef, undef );
    $tree_window->set_usize( 30, 300 );
    $tree_window->set_policy( 'automatic', 'automatic' );
    $tree_window ->add_with_viewport( $self->{tree_organisms} );
    $tree_window ->show();
    $tree_frame->add($tree_window);
    $tree_frame ->show();
    $notebook->append_page( $tree_frame ,new Gtk::Label("Seqdb by Organisms" ) );

    my $tree_frame= new Gtk::Frame();
    my $tree_window = new Gtk::ScrolledWindow( undef, undef );
    $tree_window->set_usize( 30, 300 );
    $tree_window->set_policy( 'automatic', 'automatic' );   
    $tree_window ->add_with_viewport(  $self->{tree_results} );
    $tree_window ->show();
    $tree_frame->add($tree_window);
    $tree_frame ->show();
    $notebook->append_page( $tree_frame ,new Gtk::Label("Seqdb by Results" ) );
    
    $notebook->set_page( 0 );
    $notebook->show();
    
    my $action_area=$self->action_area();
    
    my $bouton_ok=new Gtk::Button( "OK" );
    $bouton_ok->set_usize( 100, 20 );
    $bouton_ok->show();
    $action_area->pack_start($bouton_ok,$false,$false,0);
    $self->{'button_ok'} =  $bouton_ok;
    $self->{'signal_button_ok'}=>undef, 
    
    my $bouton_cancel =new Gtk::Button("Cancel");
    $bouton_cancel->set_usize( 100, 20 );
    $bouton_cancel->show();
    $bouton_cancel->signal_connect( 'clicked',sub{$self->hide();return 1;});
    $action_area->pack_start($bouton_cancel,$false,$false,0);
    
        
    bless $self,$class;
    
    return $self;  	
}
sub _load_leaves_trees{
    
    my $self=shift;
    
    my $organisms= CompAnalResultSource::SeqDBResultSource::SeqDBResultHandler->getOrganisms();
    my $elements = CompAnalResultSource::SeqDBResultSource::SeqDBResultHandler->getElements(\@$organisms);
    my $results = CompAnalResultSource::SeqDBResultSource::SeqDBResultHandler->getResultsFromElements(\@$elements );
    $self->{typeresults} = CompAnalResultSource::SeqDBResultSource::SeqDBResultHandler->getExistingResults();	
    $self->{totalresults}=[];

    foreach my $result (@$results){
	my $organism1 =undef;
	my $element1 = undef;
	ORGANISM :foreach my $organism (@$organisms){
	    if ( $organism->{organism_Id} eq $result->{organism_Id} ){
		$organism1=$organism;
		last ORGANISM;
	    }		
	}
	
	ELEMENT :foreach  my $element (@$elements){
	    if ( $element->{element_Id} eq $result->{element_Id} ){
		$element1=$element;
		last ELEMENT;
	    }
	}
	push @{$self->{totalresults}},{result=>$result,
				       organism=>$organism1,
				       element=>$element1 }; 
    }
    return 1;  
}
sub _create_tree_organisms{
    
    my $self = shift;
    my $itemname = shift;

    my $tree = new Gtk::Tree();
    my $item = new_with_label Gtk::TreeItem(  $itemname );
    my $subtree = new Gtk::Tree();
    $tree->append($item);
    $tree->set_selection_mode('single');
    $tree->set_view_mode( 'item' );
    $item->show();   
    $tree->show();  
    $item ->set_subtree( $subtree );
#   print "dir ",$item->children->get(),"\n";
   
    foreach my $result (@{$self->{totalresults}}){
	
	my $orgitem=undef;
	my $eltitem=undef;
	my @children;

	if ( $item->subtree->children ){
	    @children=$item->subtree->children;
	  ORG: while(my $itemorg =pop @children){
	      if ($itemorg->child->get eq $result->{organism}->{name}){
		  $orgitem = $itemorg;
		  last ORG;
	      }	
	  }
	}
	if (! $orgitem ){
	    $orgitem = new_with_label Gtk::TreeItem( $result->{organism}->{name} );	  
	    my $new_subtree = new Gtk::Tree();
	    $orgitem->show();
	    $item->subtree->append( $orgitem );
	    $item->subtree->set_view_mode( 'item' );  
	    $orgitem->set_subtree( $new_subtree );
	}
	
	@children=$orgitem->subtree->children;
      ELT: while(my $itemelt =pop @children){
	  if ($itemelt->child->get eq $result->{element}->{type}." ".$result->{element}->{description} ){
	      $eltitem = $itemelt;
	      last ELT;
	  }	
      }
	if( ! $eltitem ){
	    $eltitem = new_with_label Gtk::TreeItem($result->{element}->{type}." ".$result->{element}->{description} );
	    my $new_subtree = new Gtk::Tree(); 
	    $eltitem->show();
	    $orgitem->subtree->append( $eltitem );
	    $orgitem->subtree->set_view_mode( 'item' );
	    $eltitem->set_subtree( $new_subtree );	  
	}
	foreach my $typeresult(@{$self->{typeresults}}){
	    if ($typeresult->{resultType_Id} eq $result->{result}->{resultType_Id}){
		my $resultitem = new_with_label Gtk::TreeItem( $typeresult->{description} );
		$resultitem->show();
		$eltitem->subtree->append( $resultitem );
		$eltitem->subtree->set_view_mode( 'item' );
		$resultitem->signal_connect( 'select', \&_selected_item,$resultitem,$self,$orgitem->child->get,$result->{element} );
	    }
	}
    } 
    return $tree;
}	
sub _create_tree_results{
    
    my $self = shift;
    my $itemname = shift;

    my $tree = new Gtk::Tree();
    my $item = new_with_label Gtk::TreeItem(  $itemname );
    my $subtree = new Gtk::Tree();
    $tree->append($item);
    $tree->set_selection_mode('single');
    $tree->set_view_mode( 'item' );
    $item->show();   
    $tree->show();  
    $item ->set_subtree( $subtree );
#    print "dir ",$item->children->get(),"\n";
    
    my @resulttypes;
    
    foreach my $typeresult(@{$self->{typeresults}}){
	my $resultitem = new_with_label Gtk::TreeItem( $typeresult->{description} );
	$item->subtree->append($resultitem);
	$item->subtree->set_view_mode( 'item' );
	my $new_subtree = new Gtk::Tree();
	$resultitem->set_subtree( $new_subtree );
	$resultitem->show;
	push @resulttypes,{item=> $resultitem,
			   id=> $typeresult->{resultType_Id}
		       };
    }

    foreach my $result (@{$self->{totalresults}}){
	my @tabtest =@resulttypes;
	my $orgitem=undef;
	my $itemresult;
	
	TEST : foreach my $test(@resulttypes) {
	    if ($result->{result}->{resultType_Id} eq  $test->{id}){
		$itemresult = $test->{item};
		last TEST;
	    }   
	}

	if ( $itemresult->subtree->children ){
	    my @children=$itemresult->subtree->children;
	   ORG: while(my $itemorg =pop @children){
	      if ($itemorg->child->get eq $result->{organism}->{name}){
		  $orgitem = $itemorg;
		  last ORG;
	      }	
	  }
	}
	if (! $orgitem ){
	    $orgitem = new_with_label Gtk::TreeItem( $result->{organism}->{name} );	  
	    my $new_subtree = new Gtk::Tree();
	    $itemresult->subtree->append( $orgitem );
	    $itemresult->subtree->set_view_mode( 'item' );  
	    $orgitem->show();
	    $orgitem->set_subtree( $new_subtree );
	}

	if ($result->{element}->{element_Id} eq $result->{result}->{element_Id}){
	    my $eltitem = new_with_label Gtk::TreeItem($result->{element}->{type}." ".$result->{element}->{description});
	    $orgitem->subtree->append( $eltitem );
	    $orgitem->subtree->set_view_mode( 'item' );
	    $eltitem->show();
	    $eltitem->signal_connect( 'select', \&_selected_item,$itemresult,$self,$orgitem->child->get,$result->{element} );
	}	
	else {"que se passe t il \n"};
    }
    return $tree;
}

# Called whenever an item is clicked on the tree widget.
sub _selected_item{
    my ( $widget,$itemselected,$self,$orgname,$elt ) = @_;
    my $item=$itemselected->child->get;
    if($self->{signal_button_ok}){
	$self->{button_ok}->signal_disconnect($self->{signal_button_ok});
    } 
     $self->{signal_button_ok}= $self->{button_ok}->signal_connect( "clicked", sub { $self->hide(); $self->result_selected($item,$orgname,$elt); });
}

sub result_selected {
    my $self=shift;
    my $itemselected=shift;
    my $organism=shift;
    my $elt=shift;

    my $mapinfo=$self->{manager}->get_mapinfo('NAME' => $self->{mapname});
    if (defined $mapinfo) {
				
	my $source=new CompAnalResultSource::SeqDBResultSource::SeqDBCompAnalResult($itemselected,$organism,$elt);
		
	$self->{manager}->add_results_from_source($self->{mapname},$source);
    }

    return 1;
}
sub destroy{
}
1;
# END 
